import {
    Body,
    Controller,
    Get,
    HttpCode,
    HttpStatus,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { Constants } from "src/utils/constants";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";
import { UpdateServiceTypeCmsDto } from "./dto/update-service-type-cms.dto";
import { ServiceTypeCmsService } from "./service-type-cms.service";

@Controller({ path: "service-type-cms", version: Constants.API_VERSION_1 })

export class ServiceTypeCmsController {
    constructor(private readonly service: ServiceTypeCmsService) {
    }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
      [
          { name: "imageOne", maxCount: 1 },
          { name: "imageTwo", maxCount: 1 },
          { name: "imageThree", maxCount: 1 }
      ]
    ))
    updateOrCreateData(
      @Body() dto: UpdateServiceTypeCmsDto,
      @UploadedFiles() files: {
          imageOne: UploadedMulterFileI,
          imageTwo: UploadedMulterFileI,
          imageThree: UploadedMulterFileI
      }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
