import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { UpdateOrCreateStaticSiteMapCmsDto } from './dto/update-or-create-static-site-map-cms.dto';
import { StaticSiteMapCmsEntity } from './entities/static-site-map-cms.entity';
import { StaticSiteMapCmsRepository } from './static-site-map-cms.repository';
import * as fs from 'fs';

@Injectable()
export class StaticSiteMapCmsService {    
    constructor(
        @InjectRepository(StaticSiteMapCmsRepository)
        private readonly repository: StaticSiteMapCmsRepository
    ) { }

    async updateOrCreateData(dto: UpdateOrCreateStaticSiteMapCmsDto): Promise<StaticSiteMapCmsEntity | Error> {
        const isExists = await this.repository.find();
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.GENERIC_ERROR,
                    HttpStatus.FORBIDDEN
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.FORBIDDEN
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
        }
    }

    async findAll(): Promise<StaticSiteMapCmsEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    generateSitemap(dto: UpdateOrCreateStaticSiteMapCmsDto) {
        try {
            fs.writeFileSync("sitemap.xml", dto.siteMap);
        } catch (err) {
            console.error(err);
            throw err;
        }

        return ResponseUtils.successResponseHandler(200, "Sitemap updated successfully", "data", dto.siteMap);
    }
}
