import { Body, Controller, Delete, Get, Param, Patch, Post, Req, UseGuards } from "@nestjs/common";
import { AuthGuard } from "src/core/guards/auth.guard";
import { Constants } from "src/utils/constants";
import { BillingAddressService } from "./billing-address.service";
import { UpdateBillingAddressDto } from "./dto/update-billing-address.dto";

@Controller({ path: "billing-address", version: Constants.API_VERSION_1 })

export class BillingAddressController {
    constructor(private readonly service: BillingAddressService) {
    }

    @Post()
    @UseGuards(AuthGuard)
    create(
      @Body() dto: UpdateBillingAddressDto,
      @Req() request: any
    ) {
        return this.service.create(dto, request);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    // @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AuthGuard)
    update(
      @Param("id") id: string,
      @Body() dto: UpdateBillingAddressDto,
      @Req() request: any
    ) {
        return this.service.update(id, dto, request);
    }

    @Delete(":id")
    // @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
