import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateBlogRelatedPostsDto } from "./dto/create-blog-related-posts.dto";
import { BlogRelatedPosts } from "./entities/blog-related-posts.entity";

@EntityRepository(BlogRelatedPosts)
export class BlogRelatedPostsRepository extends Repository<BlogRelatedPosts> {

    async createBlogRelatedPosts(dto: CreateBlogRelatedPostsDto): Promise<BlogRelatedPosts | Error> {
        try {
            const {
                postId,
                postTitle,
                relatedPostId,
                relatedPostTitle
            } = dto;
            const entity = this.create({
                postId,
                postTitle,
                relatedPostId,
                relatedPostTitle
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async deleteBlogRelatedPostsByBlogID(postId: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
              .createQueryBuilder()
              .update(BlogRelatedPosts)
              .set({ isDeleted: true })
              .where("postId = :postId", { postId: postId })
              .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async findBlogRelatedPostsByPostId(postId: string): Promise<any> {
        try {
            return await this.find({
                where: { postId: postId, isDeleted: false },
                select: ["relatedPostId", "relatedPostTitle"]
            });
        } catch (e) {
            return e;
        }
    }
}