import { Injectable } from "@nestjs/common";
import { Cron, CronExpression } from "@nestjs/schedule";
import { InjectRepository } from "@nestjs/typeorm";
import {
    fetchAllProductCategory,
    fetchAllProductsForScheduler,
    fetchAllRetailers,
    fetchProductsCount
} from "src/helper/dutchie-plus-query";
import { getGraphQLClient } from "src/helper/graphql-client-helper";
import { CreateProductCategoryDto } from "../product-category/dto/create-product-category.dto";
import { ProductCategoryRepository } from "../product-category/product-category.repository";
import { CreateProductServiceDto } from "../product-service/dto/create-product-service.dto";
import { ProductServiceRepository } from "../product-service/product-service.repository";

@Injectable()
export class SchedulerService {
    constructor(
      @InjectRepository(ProductCategoryRepository)
      private readonly productCatRepo: ProductCategoryRepository,
      @InjectRepository(ProductServiceRepository)
      private readonly productServiceRepo: ProductServiceRepository
    ) {
    }

    // create or update dutchie plus category all retailers
    @Cron(CronExpression.EVERY_12_HOURS)
    async createUpdateDutchiePlusCategory() {
        try {
            console.log(">>>Run Create or Update Dutchie Plus Category Scheduler<<<");
            const graphQLClient = getGraphQLClient();
            const query = fetchAllRetailers();
            const data = await graphQLClient.request(query);
            const retailers = data.retailers;
            if (retailers.length > 0) {
                let category = [];
                for (let i = 0; i < retailers.length; i++) {
                    // find all category based on reatilers
                    const variables = {
                        retailerId: retailers[i].id
                    };
                    const query = fetchAllProductCategory();
                    const data = await graphQLClient.request(query, variables);
                    const categories = data["menu"]["products"];

                    // remove duplicate product category from api response
                    for (let i = 0; i < categories.length; i++) {
                        if (!category.includes(categories[i].category)) {
                            category.push(categories[i].category);
                        }
                    }
                }
                // fetch all product category
                const productCat = await this.productCatRepo.findAll();
                // console.log({productCat});
                if (Object.keys(productCat).length > 0) {
                    const difference = category.filter(
                      (obj: any) => {
                          return (productCat.findIndex((c) => c.categoryName === obj) === -1);
                      }
                    );
                    console.log("Product Categories Difference = ", difference);
                    // if difference found then create entities with difference product category
                    for (let j = 0; j < difference.length; j++) {
                        const dto = {
                            categoryName: difference[j]
                        };
                        const createProCat = await this.productCatRepo.createEntity(
                          dto as CreateProductCategoryDto
                        );
                        console.log({ createProCat });
                    }
                } else {
                    // if product categories table is empty in our local database then create all the category in local
                    for (let j = 0; j < category.length; j++) {
                        const dto = {
                            categoryName: category[j]
                        };
                        const createProCat = await this.productCatRepo.createEntity(
                          dto as CreateProductCategoryDto
                        );
                        console.log({ createProCat });
                    }
                }
            } else {
                console.log("Retailers Not Found!");
            }
        } catch (e) {
            console.log(e);
            return e;
        }
    }

    // @Cron(CronExpression.EVERY_HOUR)
    // async createUpdateDutchiePlusProducts() {
    //     try {
    //         console.log(">>>Run Create or Update Dutchie Plus Products Scheduler<<<");
    //         const graphQLClient = getGraphQLClient();
    //         const query = fetchAllRetailers();
    //         const data = await graphQLClient.request(query);
    //         const retailers = data.retailers;
    //         if (retailers.length > 0) {
    //             for (let i = 0; i < retailers.length; i++) {
    //                 // find all products count based on retailers
    //                 const variables = {
    //                     retailerId: retailers[i].id
    //                 };
    //                 const fetchProductsCountQuery = fetchProductsCount();
    //                 const productCountData = await graphQLClient.request(
    //                   fetchProductsCountQuery,
    //                   variables
    //                 );
    //                 const productCount = productCountData["menu"].productsCount;
    //                 if (productCount > 0) {
    //                     // fetch all products from dutchie plus
    //                     const productsVariable = {
    //                         retailerId: retailers[i].id,
    //                         pagination: {
    //                             offset: 0,
    //                             limit: productCount
    //                         }
    //                     };
    //                     const fetchProductsQuery = fetchAllProductsForScheduler();
    //                     const productData = await graphQLClient.request(
    //                       fetchProductsQuery,
    //                       productsVariable
    //                     );
    //                     // fetch local database data
    //                     const productService = await this.productServiceRepo.findAll();
    //                     if (Object.keys(productService).length > 0) {
    //                         // get difference between local products and dutchie products
    //                         const differenceArray = productData["menu"]["products"].filter(
    //                           (obj: any) => {
    //                               return (
    //                                 productService.findIndex((o) => o.productId === obj.id) ===
    //                                 -1
    //                               );
    //                           }
    //                         );
    //                         console.log("Product Difference=", differenceArray);
    //                         for (let j = 0; j < differenceArray.length; j++) {
    //                             const brand = differenceArray[j].brand !== null ? differenceArray[j].brand.name : "";
    //                             const productServiceDto = {
    //                                 retailerId: retailers[i].id,
    //                                 retailerName: retailers[i].name,
    //                                 productId: differenceArray[j].id,
    //                                 productName: differenceArray[j].name,
    //                                 productSlug: differenceArray[j].slug,
    //                                 image: differenceArray[j].image,
    //                                 brandName: brand,
    //                                 category: differenceArray[j].category,
    //                                 variants: differenceArray[j].variants.length
    //                             };
    //                             await this.productServiceRepo.createEntity(
    //                               productServiceDto as CreateProductServiceDto
    //                             );
    //                         }
    //                     } else {
    //                         // if no product in database create an entity
    //                         for (let k = 0; k < productData['menu']['products'].length; k++) {
    //                             const brand = productData["menu"]["products"][k].brand !== null ? productData["menu"]["products"][k].brand.name : "";
    //                             const productServiceDto = {
    //                                 retailerId: retailers[i].id,
    //                                 retailerName: retailers[i].name,
    //                                 productId: productData["menu"]["products"][k].id,
    //                                 productName: productData["menu"]["products"][k].name,
    //                                 productSlug: productData["menu"]["products"][k].slug,
    //                                 image: productData["menu"]["products"][k].image,
    //                                 brandName: brand,
    //                                 category: productData["menu"]["products"][k].category,
    //                                 variants: productData["menu"]["products"][k].variants.length
    //                             };
    //                             await this.productServiceRepo.createEntity(
    //                               productServiceDto as CreateProductServiceDto
    //                             );
    //                         }
    //                     }
    //                 }
    //             }
    //         } else {
    //             console.log("Retailers Not Found!");
    //         }
    //     } catch (e) {
    //         console.log(e);
    //         return e;
    //     }
    // }
}
